        processor 6502
        include ..\vcs.h


		;; define constants
        SEG.U defines

c_Bank0CodeArea    = $1000
cs_Bank0CodeArea   = $D000
		
c_Bank1CodeArea    = $2000
cs_Bank1CodeArea   = $F000
		
c_Bank0SwitchArea  = $1FC0
cs_Bank0SwitchArea = $DFC0
	
c_Bank1SwitchArea  = $2FC0
cs_Bank1SwitchArea = $FFC0
	
c_F8Page0          = $1FF8
c_F8Page1          = $1FF9

        
		;; define program variables
        SEG.U Bank0Vars
        ORG $80
		
v_Temp		ds	1		
v_TempStack	ds	1		
v_LoopCount	ds	1		
v_Result	ds	1		

v_TextPtr0	ds      2
v_TextPtr1	ds      2
v_TextPtr2	ds      2
v_TextPtr3	ds      2
v_TextPtr4	ds      2
v_TextPtr5	ds      2


        
        SEG  CodeBank0	
        ORG  c_Bank0CodeArea
        RORG cs_Bank0CodeArea

        .BYTE "8K Bank switching test code."
Start
        SEI		; Disable Any Interrupts 
        CLD		; Clear BCD math bit.
        LDX #$FF	; put X to the top...
        TXS		; ...and use it reset the stack pointer


        ;;  clear the memory and TIA registers
        LDX #$FF	;X is top of memory
        LDA #0		;Put Zero into A
ClearMem 
        STA 0,X		;Now, this doesn't mean what you think...
        DEX		;decrement X (decrease X by one)
        BNE ClearMem

        JSR l_ProgInit	; Init program's variables

		
l_MainLoop
        JSR l_VerticalBlank     ; Execute the vertical blank code.
        JSR l_BankSwitchingTest ; Jumps between the banks and updated v_Result
        JSR l_ProcessResult     ; Checks the result and sets the report to print
        JSR l_DrawScreen        ; Draw the screen
        JSR l_OverScan          ; Do Overscan calculations
        JMP l_MainLoop          ; Continue this loop forver! Back to the code for the vsync etc



;*********************** 
l_ProgInit		
;*********************** 
	;First lets make the background black
        LDA #$00	
        STA COLUBK	

	;Player0 color setting
        LDA #53
	STA COLUP0

        LDA #$01
        STA CTRLPF
        
        ;; Clear text pointers
        LDX #11
        LDA #00
l_ClrPtrLp
        STA v_TextPtr0,X
        DEX
        BPL l_ClrPtrLp
        
        STA WSYNC		; Wait till the end of the scan line (#001)
        LDY #03			; Careate some delay after the beginning of
				; line so the text will appear in the center.
l_Pos0
        DEY
        BNE l_Pos0
        STA RESP0		; Reset P0 location
        
	RTS			; Return to caller

        
;*********************** 
l_VerticalBlank
;*********************** 
;
        LDA  #2
        STA  VSYNC	; Start the sync up by setting D1 of VSYNC 
			
        STA  WSYNC	; Wait for 3 lines... (#002) 
        STA  WSYNC 	;                     (#003)
        STA  WSYNC	;                     (#004)

;	So how much time will those 37 scan lines take?
;	Each scanline takes 76 cycles (which are the same thing our clock is geared to)
;	37*76 = 2812 (no matter what Nick Bensema tries to tell us...his "How to Draw 
;	A Playfield" is good in a lot of other ways though..to quote the comments from
;	that:
;		 We must also subtract the five cycles it will take to set the
;		 timer, and the three cycles it will take to STA WSYNC to the next
;		 line.  Plus the checking loop is only accurate to six cycles, making
; 		a total of fourteen cycles we have to waste.  
;
;	So, we need to burn 2812-14=2798 cycles.  Now, there are a couple of different
;	timers we can use, and Nick says the one we usually use to make it work out right
;	is TIM64T, which ticks down one every 64 cycles.  2798 / 64 = 43.something,
;	but we have to play conservative and round down.
;

        LDA  #43	;load 43 (decimal) in the accumulator
        STA  TIM64T	;and store that in the timer

        LDA #0		;Zero out the VSYNC
        STA  VSYNC 	; 'cause that time is over

;
;	So here we can do a ton of game logic, and we don't have
;	to worry too much about how many instructions we're doin,
;	as long as it's less than 37 scanlines worth (if it's not
;	less, your program is screwed with a capital screw)
        RTS



;*********************** 
l_BankSwitchingTest
;*********************** 
	JSR l_Jump2Page1 	; Switch to bank 1 (currently at running in 0)
        RTS

;*********************** 
l_ProcessResult
;*********************** 
	;; Set Character #0
        LDA #<l_P
        STA v_TextPtr0
        LDA #>l_P
        STA v_TextPtr0+1
	
	;; Set Character #1
        LDA #<l_A
        STA v_TextPtr1
        LDA #>l_A
        STA v_TextPtr1+1
        
	;; Set Character #2
        LDA #<l_S
        STA v_TextPtr2
        LDA #>l_S
        STA v_TextPtr2+1
        
	;; Set Character #3
        LDA #<l_S
        STA v_TextPtr3
        LDA #>l_S
        STA v_TextPtr3+1
        
	;; Set Character #4
        LDA #<l_Blank
        STA v_TextPtr4
        LDA #>l_Blank
        STA v_TextPtr4+1
        
	;; Set Character #5
        LDA #<l_Blank
        STA v_TextPtr5
        LDA #>l_Blank
        STA v_TextPtr5+1
                
        RTS			; Return to main loop


;*********************** 
l_DrawScreen
;***********************
;*********************** VERTICAL BLANK WAIT-ER
		
WaitForVblankEnd
       LDA INTIM		;load timer...
       BNE WaitForVblankEnd	;killing time if the timer's not yet zero
				
				; A = 0 at this point 
       STA WSYNC		; Wait till the end of the line (#004)
       STA HMOVE		; Stop any horizontal movement
       STA VBLANK		; End the VBLANK period with the zero (again ?!)
       STA WSYNC		; Wait till the end of the line (#005)
	
        ;; From here, we are at the begining of the frame

        LDA #254                ; Set the color of the text (again?)
	STA COLUP0
	STA COLUP1

        LDX #03			; Set the size and appearance of the players 
        STX NUSIZ0
        STX NUSIZ1 

        STA WSYNC		; Wait for the previous line to finish (#006)

;*********************** Scan line Loop
ScanLoop 
	;; Wait for the proper place for the text location
        ldx #22
l_LocLp0    
	dex
        bne l_LocLp0
        sta RESP0		; Reset P0

        sta WSYNC		; Wait for the previous line to finish (#007)
        ldx #22
l_LocLp1   
	dex
        bne l_LocLp1
	
        nop
        sta RESP1		; Reset P1
        sta HMCLR		; 
        lda #%10100000;2 more than belo
        sta HMP1
        lda #%11000000;
        sta HMP0
        sta WSYNC
        sta HMOVE
        STA WSYNC       ;Wait for the previous line to finish

        LDA #$07
        STA v_LoopCount

        LDX #$09
l_Pause1
        DEX
        BNE l_Pause1
        NOP
        NOP
        NOP
        
        TSX
        STX v_TempStack
l_DrawText
        LDY v_LoopCount
        LDA (v_TextPtr0),Y
        STA GRP0
        
        BIT $00
        
        LDA (v_TextPtr1),Y
        STA GRP1
        
        LDA (v_TextPtr5),Y
        TAX
        TXS
        
        LDA (v_TextPtr2),Y
        STA v_Temp
                        
        LDA (v_TextPtr3),Y
        TAX 
                        
        LDA (v_TextPtr4),Y

        LDY v_Temp
        STY GRP0
        STX GRP1
        STA GRP0
        TSX
        STX GRP1

        DEC v_LoopCount
        BPL l_DrawText

        LDX v_TempStack
        TXS

        LDA #$00
        STA GRP0
        STA GRP1

        
        LDA #2		;#2 for the VBLANK...
        STA WSYNC  	;Finish this final scanline.
        STA VBLANK 	; Make TIA output invisible for the overscan, 
			; (and keep it that way for the vsync and vblank)


        LDY #0
        STY PF0
        STY PF1
        STY PF2
        
        RTS
        
;***********************
l_OverScan        
;***********************
;***************************** OVERSCAN CALCULATIONS
;
;I'm just gonna count off the 30 lines of the overscan.
;You could do more program code if you wanted to.


        LDA #35		;store 30
        STA TIM64T
        
OverScanWait
        LDA INTIM
        BNE OverScanWait

        RTS


	include ../letters.s
	
	
        
        

	ORG  c_Bank0SwitchArea
	RORG cs_Bank0SwitchArea
	
;*********************** 
l_Jump2Page1
;*********************** 
	LDA c_F8Page1		; Present $1FF9 on the address bus
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
		
	RTS


	ORG $1FFC
	RORG $DFFC
;*********************** 
; Last addresses of Bank0
;*********************** 
	.word Start
	.word Start


;*********************** 
;  Start of 2nd bank
;*********************** 

	SEG.U Bank1Vars
	ORG $80

dummy1 ds 1
	
	SEG	CodeBank1
	ORG     c_Bank1CodeArea
	RORG	cs_Bank1CodeArea

;*********************** 
l_page1Start
;*********************** 
	; Here we have almost 4KB of code to work with...
	NOP			; Just a place holder
	NOP			; Just a place holder
	RTS			; Return



	
	ORG	c_Bank1SwitchArea
	RORG	cs_Bank1SwitchArea
;*********************** 
l_Bank1SwitchArea
;*********************** 
	NOP
	NOP
	NOP
	JSR	l_Page1Start
	LDA	c_F8Page0	; Present $1FF8 on the address bus 
	NOP

	ORG $2FFC
	RORG $FFFC
;*********************** 
; Last addresses of Bank1
;*********************** 
	.word l_page1Start
	.word l_page1Start

    
